' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE "SDL.INC"
#INCLUDE "GLU.INC"
#INCLUDE "GDIPUTILS.INC"

$WindowCaption = "NeHe Lesson 6"

GLOBAL TextureHandle AS DWORD
GLOBAL xrot AS SINGLE
GLOBAL yrot AS SINGLE
GLOBAL zrot AS SINGLE

' =======================================================================================
' All the setup goes here
' =======================================================================================
SUB SetupScene ()

	LOCAL hr AS LONG
	LOCAL strTextureData AS STRING
	LOCAL TextureWidth, TextureHeight AS LONG

	' Specify clear values for the color buffers
	glClearColor 0.0!, 0.0!, 0.0!, 0.0!
	' Specify the clear value for the depth buffer
	glClearDepth 1.0!
	' Specify the value used for depth-buffer comparisons
	glDepthFunc %GL_LESS
	' Enable depth comparisons and update the depth buffer
	glEnable %GL_DEPTH_TEST
	' Select smooth shading
	glShadeModel %GL_SMOOTH

	' Load bitmap texture from disk
	hr = GdiPlusLoadTexture("NeHe.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

	' Assign an OpenGL handle to this texture
	glGenTextures 1, TextureHandle

	' Activate our newly created texture
	glEnable %GL_TEXTURE_2D
	glTexEnvf %GL_TEXTURE_ENV, %GL_TEXTURE_ENV_MODE, %GL_MODULATE
	glBindTexture %GL_TEXTURE_2D, TextureHandle

	' Create the texture
	glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
	glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
	glTexImage2D %GL_TEXTURE_2D, 0, %GL_RGBA, TextureWidth, TextureHeight, 0, _
					 %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

END SUB
' =======================================================================================

' =======================================================================================
' Draw the scene
' =======================================================================================
SUB DrawScene (BYVAL nWidth AS LONG, BYVAL nHeight AS LONG)

	' Clear the screen buffer
	glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

	' Select and setup the projection matrix
	glMatrixMode %GL_PROJECTION
	glLoadIdentity

	' Prevent divide by zero making height equal one
	IF nHeight = 0 THEN nHeight = 1

	' Calculate the aspect ratio of the window
	gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

	' Select and setup the modelview matrix
	glMatrixMode %GL_MODELVIEW
	glLoadIdentity

	glTranslatef 0.0!, 0.0!, -5.0!
	glRotatef xrot, 1.0!, 0.0!, 0.0!			  ' Rotate on the x axis
	glRotatef yrot, 0.0!, 1.0!, 0.0!			  ' Rotate on the y axis
	glRotatef zrot, 0.0!, 0.0!, 1.0!			  ' Rotate on the z axis
	glBindTexture %GL_TEXTURE_2D, TextureHandle

	glBegin %GL_QUADS
		' Front Face
		glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!		' bottom left of the texture and quad
		glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!		' bottom right of the texture and quad
		glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!		' top right of the texture and quad
		glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!		' top left of the texture and quad
		' Back Face
		glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!		' bottom right of the texture and quad
		glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!		' top right of the texture and quad
		glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!		' top left of the texture and quad
		glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!		' bottom left of the texture and quad
		' Top Face
		glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!		' top left of the texture and quad
		glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!,  1.0!,  1.0!		' bottom left of the texture and quad
		glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!,  1.0!,  1.0!		' bottom right of the texture and quad
		glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!		' top right of the texture and quad
		' Bottom Face
		glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!, -1.0!, -1.0!		' top right of the texture and quad
		glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!, -1.0!, -1.0!		' top left of the texture and quad
		glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!		' bottom left of the texture and quad
		glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!		' bottom right of the texture and quad
		' Right face
		glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!		' bottom right of the texture and quad
		glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!		' top right of the texture and quad
		glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!		' top left of the texture and quad
		glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!		' bottom left of the texture and quad
		' Left Face
		glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!		' bottom left of the texture and quad
		glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!		' bottom right of the texture and quad
		glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!		' top right of the texture and quad
		glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!		' top left of the texture and quad
	glEnd

	xrot = xrot + 0.3!								 ' x axis rotation
	yrot = yrot + 0.2!								 ' y axis rotation
	zrot = zrot + 0.4!								 ' z axis rotation

	' // Swap buffers
	SDL_GL_SwapBuffers

END SUB
' =======================================================================================

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

	LOCAL pscreen AS SDL_Surface PTR
	LOCAL nWidth, nHeight, bpp AS LONG
	LOCAL lResult, bFullScreen AS LONG
   LOCAL szCaption AS ASCIIZ * 256
   LOCAL dwFlags AS DWORD
   LOCAL t, t0, fps AS DOUBLE
   LOCAL nFrames AS LONG

   szCaption = $WindowCaption   ' Window caption

	' Ask the user which screen mode he prefers
	lResult = MessageBox(%NULL, "Would you like to run in fullscreen mode?", _
				 "Start fullScreen?", %MB_YESNOCANCEL OR %MB_ICONQUESTION)
	SELECT CASE lResult
		CASE %IDCANCEL : EXIT FUNCTION
		CASE %IDYES	 : bFullScreen = %TRUE
		CASE %IDNO	  : bFullScreen = %FALSE
	END SELECT

	' // Initialize SDL's subsystems - in this case, only video.
	IF SDL_Init(%SDL_INIT_VIDEO) < 0 THEN
		MSGBOX "Unable to init SDL: " & SDL_GetError()
		EXIT FUNCTION
	END IF

	' // Attempt to create a 640x480 window
	nWidth = 640 : nHeight = 480 : bpp = 16
	IF bFullScreen THEN
		dwFlags = %SDL_OPENGL OR %SDL_FULLSCREEN
	ELSE
		dwFlags = %SDL_OPENGL OR %SDL_RESIZABLE
	END IF
	pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
	SDL_WM_SetCaption szCaption, ""

	' // If we fail, return error.
	IF pscreen = %NULL THEN
		SDL_Quit
		MSGBOX "Unable to set the video mode: " & SDL_GetError()
		EXIT FUNCTION
	END IF

	' // Set up the scene
	SetupScene

	' // Main loop
	LOCAL done AS LONG
	LOCAL uevent AS SDL_Event
	WHILE done = %FALSE

		' // Poll for events, and handle the ones we care about.
		WHILE SDL_PollEvent(VARPTR(uevent))
			SELECT CASE uevent.type
				CASE %SDL_KEYDOWN
				CASE %SDL_KEYUP
					' // Quit if escape is pressed
					IF uevent.key.keysym.sym = %SDLK_ESCAPE THEN
						done = %TRUE
						EXIT LOOP
					END IF
				CASE %SDL_VIDEORESIZE
					' // Change the video mode to the new window size
					nWidth = uevent.resize.w
					nHeight = uevent.resize.h
					pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
					IF TextureHandle THEN
						glDeleteTextures(1, TextureHandle)
						TextureHandle = 0
					END IF
					SetupScene
				CASE %SDL_QUIT
					done = %TRUE
					EXIT LOOP
			END SELECT
		WEND

      IF ISFALSE bFullScreen THEN
         ' Get time and mouse position
         t = INT(TIMER)
         ' Calculate and display FPS (frames per second)
         IF t > t0 OR nFrames = 0 THEN
            fps = nFrames \ (t - t0)
            wsprintf szCaption, $WindowCaption & " (%i FPS)", BYVAL fps
            SDL_WM_SetCaption szCaption, szCaption
            t0 = t
            nFrames = 0
         END IF
         nFrames = nFrames + 1
      END IF

		' // Draw the scene
		DrawScene nWidth, nHeight

	WEND

	' Delete the texture
	IF TextureHandle THEN glDeleteTextures(1, TextureHandle)

	' Shut down SDL
	SDL_Quit

END FUNCTION
' =======================================================================================
